'use strict'
let Q = require('q')
let express = require('express')
let router = express.Router()
let _ = require('lodash')
let commons = require('@dct/commons')
let log = commons.Log.getLogger(__filename)
let _Error = commons._Error
let Response = commons.Rest.Response

const RegulareCheck = require('../lib/services/regular-inspect')
const regularCheck = RegulareCheck.regularCheckWithTask
const TaskExecutionMap = RegulareCheck.TaskExecutionMap
const CronJobService = require('../lib/services/cron-job')
const uuid = require('uuid')
const Forever = commons.Task.Forever
/**
 * 传入巡检规则id，进行立即巡检
 */
router.post('/:taskId', function (req, res) {
  let task = req.body
  Q()
    .then(function () {
      // check argument
      if (!task || !_.isObject(task)) {
        throw new _Error('bad request. missing Regular check task data in body')
      }
    })
    .then(function () {
      const executionId = uuid.v4()
      Forever.untilResolve(() => {
        return regularCheck(task, executionId)
      }, { times: 3, interval: 3000, timeout: -1 })
      return executionId
    })
    .then(function (created) {
      Response.succeed(res, created)
    })
    .catch(function (err) {
      Response.fail(res, err)
    })
})

router.get('/:taskId/:executionId', function (req, res) {
  const taskId = req.params.taskId
  const executionId = req.params.executionId
  Q()

    .then(function () {
      return TaskExecutionMap.get(taskId + '-' + executionId)
    })
    .then(function (created) {
      Response.succeed(res, created)
    })
    .catch(function (err) {
      Response.fail(res, err)
    })
})

/**
 * 传入巡检规则，根据巡检规则，创建cron job
 */
router.post('/', (req, res) => {
  let task = req.body
  Q()
    .tap(function () {
      // check argument
      if (!task || !_.isObject(task)) {
        throw new _Error('bad request. missing Regular check task data in body')
      }
    })
    .then(function () {
      return CronJobService.createCronJob(task)
    })
    .then(function (created) {
      Response.succeed(res, created)
    })
    .catch(function (err) {
      Response.fail(res, err)
    })
})
/**
 * 根据传入的taskId,更新已经创建的CronJob 并重新启动
 */
router.put('/:taskId', (req, res) => {
  const taskId = req.params.taskId
  const task = req.body
  Q()
    .tap(function () {
      // check argumen
      if (!task || !_.isObject(task)) {
        throw new _Error('bad request. missing Regular check task data in body')
      }
    })
    .then(function () {
      return CronJobService.updateCronJob(taskId, task)
    })
    .then(function (created) {
      Response.succeed(res, created)
    })
    .catch(function (err) {
      Response.fail(res, err)
    })
})

/**
 * 传入删除的taskId，停止正在执行的CronJob并删除
 */
router.delete('/', (req, res) => {
  const taskIds = req.body
  Q()

    .then(function () {
      return CronJobService.deleteCronJobsByTaskIds(taskIds)
    })
    .then(function (created) {
      Response.succeed(res, created)
    })
    .catch(function (err) {
      Response.fail(res, err)
    })
})
module.exports = router
